import sys
import os
import copy
import subprocess
import re
import argparse
from chembl_structure_pipeline import standardizer
from chembl_structure_pipeline import checker
from rdkit import Chem
from rdkit import RDLogger
from rdkit.Chem import MolStandardize
from os import path


def parse_infile(infile):
    in_ext = os.path.splitext(args.inputfile)[1][1:]
    RDLogger.DisableLog("rdApp.*")
    if in_ext.lower() == "smi":
        molecules = Chem.SmilesMolSupplier(
            infile,
            delimiter="\t",
            smilesColumn=0,
            nameColumn=1,
            titleLine=False,
            sanitize=True,
        )
    elif in_ext.lower() == "sdf":
        molecules = Chem.ForwardSDMolSupplier(infile, sanitize=True, removeHs=True)
    return molecules


def standardize_mol(molecule):
    desalter = MolStandardize.fragment.LargestFragmentChooser()
    std1_mol = standardizer.standardize_mol(molecule)
    desalt_mol = desalter.choose(std1_mol)
    std2_mol = standardizer.standardize_mol(desalt_mol)
    return std2_mol


def process_mols(mols, outfile):
    out_ext = os.path.splitext(outfile)[1][1:]

    if out_ext == "smi":
        out = Chem.rdmolfiles.SmilesWriter(
            outfile, delimiter="\t", includeHeader=False, nameHeader="_Name"
        )
    elif out_ext == "sdf":
        out = Chem.rdmolfiles.SDWriter(outfile)

    for mol in mols:
        if mol is None:
            continue
        molname = mol.GetProp("_Name")
        try:
            curated_mol = standardize_mol(mol)
        except:
            print("error in %s, skipping current compound" % molname)
            continue

        curated_mol.SetProp("_Name", molname)
        out.write(curated_mol)

    out.close()


if __name__ == "__main__":
    p = argparse.ArgumentParser(description="This is a script for compound standardization.")
    p.add_argument("inputfile", help="Input SDF or SMILES")
    p.add_argument("outputfile", help="Output SDF or SMILES")
    args = p.parse_args()

    molecules = parse_infile(args.inputfile)
    process_mols(molecules, args.outputfile)
